# Copyright (C) 2009 Ferdinand Majerech
# This file is part of IMGCrush
# For conditions of distribution and use, see copyright notice in __init__.py

from util import *
from tempimage import *
from referencefile import *

class FileTester:
    """Tests the generated files and selects which one should be output.

    Removes other temporary files in the process.
    """
    def __init__(self):
        """Constructor.

        Data members:
        Mode       : Mode of file selection. available modes:
                     "q" : choose the smallest file w/ at least given quality.
                     "s" : choose the best file with at most given filesize.
        MinQuality : Minimum quality - used when mode is q . Specified in %,
                     e.g. "10.1%" or in subjective units, e.g. "10.1", that 
                     are closer to JPG quality settings.
        MaxSize    : Maximum filesize - used when mode is s . Specified in %
                     of input filesize, e.g. "10.1%", % of filesize of the
                     reference file, e.g. "10.1r%", kiB, MiB, GiB, e.g. 
                     "10.1kiB", in bytes, e.g. "10342" or in average bits per
                     pixel, e.g. "2.4bpp" .
        FileList   : List of all temporaty images. Contains their filenames,
                     filesizes, quality values and data about how they were
                     saved.
        """
        self.Mode = "q"
        self.MinQuality = "100%"
        self.MaxSize = "50%"
        self.FileList = []
    
    def set_mode(self, m):
        """Sets mode of file selection.

        Arguments:
        m: Mode to set. Valid values are "q" and "s"
        """
        if(m in("q", "s")):
            self.Mode = m
        else:
            print "WARNING: wrong value passed by --mode (-m): using default"

    def set_min_quality(self, q):
        """Sets minimum quality used in "q" mode.

        Argumets:
        q: quality to set. Valid values are floats 0-100% or 0-100.
        """
        if q[-1] == "%":
            self.MinQuality = valid_number_str_suffix(q, "100%", "%", 
                                                      "--minquality", 0, 100,
                                                      True)      
        else:
            self.MinQuality=valid_number_str(q, "100%", "--minquality", 0, 100, 
                                             True)

    def set_max_size(self, s):
        """Sets maximum size used in "s" mode.

        Arguments:
        s: size to set. Valid values are floats 0-100% , 0-100r% , 
           0-1048576kiB/MiB/GiB , 0-1073741824
        """
        if s.endswith("r%"):
            self.MaxSize = valid_number_str_suffix(s, "50%", "r%", "--maxsize",
                                                   0, 100, True)
            print self.MaxSize
        elif s[-1]=="%":
            self.MaxSize = valid_number_str_suffix(s, "50%", "%", "--maxsize",
                                                   0, 100, True)
        elif s.endswith("bpp"):
            self.MaxSize = valid_number_str_suffix(s, "50%", "bpp", "--maxsize"
                                                   , 0, 1024, True)
        elif s.endswith("kiB") or s.endswith("MiB") or s.endswith("GiB"):
            self.MaxSize = valid_number_str_suffix(s, "50%", s[-3 :], 
                                                   "--maxsize", 0, 1048576, 
                                                   True)
        else:
            self.MaxSize = valid_number_str(s, "50%", "--maxsize", 0, 
                                            1073741824, True)
    def abs_min_quality(self):
        """Returns minimum quality in %.

        Parses stored minimum quality data and returns it's equivalent in %.
        """
        if self.MinQuality[-1] == "%":
            return float(self.MinQuality[:-1])
        else:
            #this is... crude.
            return 100.0 * sqrt(sqrt(sqrt(sqrt(float(self.MinQuality)/100))))
    
    def abs_max_size(self, reffile):
        """Returns maximum size in bytes.

        Parses stored maximum size data and returns it's equivalent in bytes.
        
        Arguments:
        reffile : Reference file used for relative max size options (i.e. in %)
        """
        if self.MaxSize.endswith("kiB"):
            return int(float(self.MaxSize[:-3]) * 1024)
        elif self.MaxSize.endswith("MiB"):
            return int(float(self.MaxSize[:-3]) * 1024 * 1024)
        elif self.MaxSize.endswith("GiB"):
            return int(float(self.MaxSize[:-3]) * 1024 * 1024 * 1024)
        elif self.MaxSize.endswith("r%"):
            return int(float(self.MaxSize[:-2]) / 100.0 * reffile.Size)
        elif self.MaxSize[-1]=="%":
            return int(float(self.MaxSize[:-1]) / 100.0 * reffile.InputSize)
        elif self.MaxSize.endswith("bpp"):
            return int(reffile.Width * reffile.Height * 
                       float(self.MaxSize[:-3]) / 8.0)
        else:
            return int(self.MaxSize)
  

    def compare_files(self, minqual, maxsize, file, bestfile):
        """Compares a file with the best file so far.

        Deletes the file that is worse in the process.

        Returns true if the file is better than the best file yet.
        Returns false otherwise.
        """
        fsize = file.Size
        fqual = file.Quality
        if((fqual >= minqual) and (fsize <= maxsize)):
            if bestfile is not None:
                subprocess.call("rm " + bestfile.FileName, shell = True)
            return True
        else:
            subprocess.call("rm " + file.FileName, shell = True)
            return False

    def select_best_file(self, filelist, reffile, outfilename):
        """Selects which of generated files should be output, deletes others.

        Arguments:
        filelist    : List of generated temporary files to select from.
        reffile     : Data about the current reference file to compare to.
        outfilename : Filename the output will have.
        """
        print "\nselecting the best file"
        self.FileList = filelist
        bestfile = None
        if self.Mode == "q":
            bestsize = reffile.Size
            minquality = self.abs_min_quality()
            for f in self.FileList:
                if self.compare_files(minquality, bestsize, f, bestfile):
                    bestfile = f
                    bestsize = f.Size
        elif self.Mode == "s":
            bestquality = 0.0
            maxsize = self.abs_max_size(reffile)
            for f in self.FileList:
                if self.compare_files(bestquality, maxsize, f, bestfile):
                    bestfile = f
                    bestquality = f.Quality
        if bestfile is not None:
            subprocess.call("mv " + bestfile.FileName + " " + outfilename + "."
                            + bestfile.FileType, shell = True)
        else:
            print "WARNING: File" + reffile.InputName + "could not be"
            print "optimized according to input. Outputting reference"
            print "file instead. This might be caused by a too small size"
            print "requirement or by requiring lossless compression"
            print "while only using jpg."
            subprocess.call("cp " + reffile.Name + " " + outfilename + ".png"\
                            , shell = True)
