# Copyright (C) 2009 Ferdinand Majerech
# This file is part of IMGCrush
# For conditions of distribution and use, see copyright notice in __init__.py

from util import *
from help import *
from referencefile import *
from jpg import *
from png import *
from gif import *
from filetester import *

class FileAddress:
    """Stores file address for an input file.
    """

    def __init__(self, infname, reffname, outfname):
        """Constructor.
         
        Data members:
        In  : Filename of the input file.
        Ref : Filename of the reference temporary file relative to temp dir.
        Out : Filename of the output file relative to output dir.
        """
        self.In = infname
        self.Ref = reffname
        self.Out = outfname


#TODO : long-term : use advpng and perhaps some other tools 
class ImageOptimizer:
    """The main class of IMGCrush.

    Handles input and high-level commands to other modules.
    """
    def __init__(self, argv):
        """Constructor.
        
        Initializes the optimizer and its modules and processes input.

        Data members:

        Files and directories:
        TempDir  : Directory used to store temporaty files.
        SaveDir  : Directory to save output files.
        FileList : Addresses of all input files.

        Resizing:
            By default, resizing preserves aspect ratio, therefore Width and
            Height are maximum bounds of the resized image. If only one
            bound is specified, e.g. Width, the image is resized to fill
            the given bound and the other is resized accordingly to preserve
            aspect ratio. If no bounds are specified or are 0, resizing is 
            turned off. Other members modify resizing behavior. 
        Width           : Maximum width of resized output.
        Height          : Maximum height of resized output.
        Fill            : If given bounds aren't filled by resized image, fill
                          the rest with background color, centering the image.
        Stretch         : If given bounds aren't filled by resized image,
                          stretch the image to fill them.
        BackgroundColor : Color used with Fill.
        
        File format modules:
        UseJPG : Use the JPG module? I.e. consider JPG output for optimization.
        UsePNG : Use the PNG module? I.e. consider PNG output for optimization.
        UseGIF : Use the GIF module? I.e. consider GIF output for optimization.
        JPG    : Object handling JPG files and their features.
        PNG    : Object handling PNG files and their features.
        GIF    : Object handling GIF files and their features.

        Output file selection:
        Tester : Object handling selection of the best file from temp files.
        """
        self.TempDir = "/dev/shm/imgcrush/"
        self.SaveDir = "./optimized/"
        self.FileList = []
        self.Width = ""
        self.Height = ""
        self.Fill = False
        self.Stretch = False
        self.BackgroundColor = "none"
        self.JPG = JPGProcessor(self.TempDir)
        self.UseJPG = False
        self.PNG = PNGProcessor(self.TempDir)
        self.UsePNG = False
        self.GIF = GIFProcessor(self.TempDir)
        self.UseGIF = False
        self.Tester = FileTester()
        self.process_args(argv)
   
    def add_file(self, f, r, o):
        """Adds an input file to the list of files to process.

        Ignores the file if it's invalid.
        
        Arguments:
        f : Filename of the input file. 
        r : Filename of the corresponding temporary reference file
            relative to the temp directory.
        o : Filename of the corresponding output file relative to the optput
            directory.
        """
        #If ImageMagick can read a file, output of identify starts by filename
        #, otherwise ignore the file
        if run_bash("identify " + escape_chars_bash(f))[ :len(f)] == f:
            self.FileList.append(FileAddress(escape_chars_bash(f), 
                                             escape_chars_bash(r),
                                             escape_chars_bash(o)))
        else :
            print "unreadable file: " + f + " :ignoring"
                                                    
      #TODO: low-priority : Recursive directories as an option
    def add_dir(self, dirname):
        """Adds all image files in a directory to the list of files to process.

        Does not add files in subdirectories.

        Arguments:
        dirname : Name of the directory to add files from.
        """
        #always end dir name with / so we can append file name to it
        if dirname[-1] != "/":
            dirname = dirname + "/"
        dirls = run_bash("ls " + dirname)
        #if the directory is empty, ignore it
        if(dirls == ""):
            return
        #strip parent dirs to use this as a subdir of temp and save dirs
        dirstrip = dirname
        if(dirstrip.rfind("/", 0, -1) >= 0):
            dirstrip = dirstrip[dirstrip.rfind("/", 0, -1) + 1: ]
        #iterate through files in the directory
        for f in dirls.split("\n"): 
            #strip filename extension
            fstrip = f
            if(fstrip.rfind(".") > 0):
                fstrip = f[ :fstrip.rfind(".")]
            outfname=dirstrip+fstrip
            self.add_file(dirname + f, outfname + "-ref.png", outfname)
        #make corresponding temp and output directories
        subprocess.call("mkdir -p " + self.TempDir + dirstrip , shell = True)
        subprocess.call("mkdir -p " + self.SaveDir + dirstrip , shell = True)
    
    def clean_up(self):
        """Cleans the temporary directory when the program finishes work.
        """
        subprocess.call("rm -r " + self.TempDir, shell = True)
    
    def process_files(self):
        """Generates temp files, selects the best and cleans up.

        This is the main method of imgcrush: 
        it handles generating files based on input using file format modules,
        then compares them using tester and outputs the best file.
        In the end it deletes the temp directory along with temporary files.
        """
        for file in self.FileList:
            reffilename = self.TempDir + file.Ref
            outfilename = self.SaveDir + file.Out
            reffile = ReferenceFile(file.In, reffilename, self.Width, 
                                    self.Height, self.Fill, self.Stretch, 
                                    self.BackgroundColor)
            outfilelist = []
            if self.UseJPG :
                outfilelist.extend(self.JPG.process_jpg(reffile, file.Out))
            if self.UsePNG :
                outfilelist.extend(self.PNG.process_png(reffile, file.Out))
            if self.UseGIF :
                outfilelist.extend(self.GIF.process_gif(reffile, file.Out))
            self.Tester.select_best_file(outfilelist, reffile, outfilename)
            
            subprocess.call("rm " + reffile.Name, shell = True)
        self.clean_up()
    
    #TODO : profile, then obvious basic optimizations 
    #       (i.e. ignoring jpg at lossless, etc.), then source package, 
    #       then advanced quality measurement, then advanced optimization 
    #       based on stats about processed files                      
    def process_args(self,argv):
        """Processes all command-line input.

        Input is validated in objects that are affected by it.
        i.e. , input that sets properties of ImageOptimizer is validated here,
        other input is passed to objects that use it and validated there.
        If an argument given to command line option is wrong, default value
        is usually used.
        If an unrecognizable option is given or an option that requires
        an argument is given without an argument, help is displayed and
        the program is aborted.

        Arguments:
        argv : List of command line options and their arguments.
        """
        opts = []
        args = []
        try:                                
            opts, args = getopt.getopt(argv, "jpgw:h:f:s:c:m:o:t:Q:S:", 
                                       ["jpg", "png", "gif", "width=", 
                                       "height=", "fill", "stretch", 
                                       "bgcolor=", "mode=", "outdir=",
                                       "tempdir=", "minquality=", "maxsize=",
                                       "help", "jpgqual=", "jpgstep=",
                                       "jpginterlace", "jpgoptimize",
                                       "jpgsampfactors=", "pnglev=", 
                                       "pngfilt=", "pngpal=", "pnggray=",
                                       "pngbitmap", "pnginterlace",
                                       "gifinterlace"]) 
        except getopt.GetoptError:
            command_line_help()                          
            sys.exit(2)  
        
        for opt, arg in opts:  
            if opt in ("-w", "--width"):
                if(arg[-1] == "%"):
                    self.Width = valid_number_str_suffix(arg, "", "%", 
                                                         "--width (-w)", 0, 
                                                         1048576, True)
                else:
                    self.Width = valid_number_str(arg, "", "--width (-w)", 1,
                                                  1048576)
                if self.Width == "0" or self.Width == "0%":
                    self.Width=""
            elif opt in ("-h", "--height"): 
                if(arg[-1]=="%"):
                    self.Height = valid_number_str_suffix(arg, "", "%", 
                                                          "--height (-h)", 0, 
                                                          1048576, True)
                else:
                    self.Height = valid_number_str(arg, "", "--height (-h)", 
                                                   1,1048576)
                if self.Height == "0" or self.Height == "0%":
                    self.Height=""
            elif opt in ("-f", "--fill"): 
                self.Fill = True 
            elif opt in ("-s", "--stretch"): 
                self.Stretch = True    
            #TODO: low-priority: Check that input from --bgcolor is recognizable by ImageMagick (at the moment, a wrong value will cause imagemagick error)
            elif opt in ("-c", "--bgcolor"): 
                self.BackgroundColor = arg
            elif opt in ("-j", "--jpg"): 
                self.UseJPG = True
            elif opt in ("--jpgqual"): 
                self.JPG.set_min_quality(arg)
            elif opt in ("--jpgstep"): 
                self.JPG.set_step(arg)
            elif opt in ("--jpginterlace"):
                self.JPG.set_interlace()
            elif opt in ("--jpgoptimize"):
                self.JPG.set_optimize()
            elif opt in ("--jpgsampfactors"):
                self.JPG.set_sfactors(arg)
            elif opt in ("-p", "--png"): 
                self.UsePNG = True      
            elif opt in ("--pnglev"): 
                self.PNG.set_min_level(arg)
            elif opt in ("--pngfilt"): 
                self.PNG.set_filters(arg)
            elif opt in ("--pngpal"): 
                self.PNG.set_min_pal_depth(arg)
            elif opt in ("--pnggray"): 
                self.PNG.set_min_gray_depth(arg)
            elif opt in ("--pngbitmap"): 
                self.PNG.set_bit_map()
            elif opt in ("--pnginterlace"): 
                self.PNG.set_interlace()
            elif opt in ("-g", "--gif"): 
                self.UseGIF = True              
            elif opt in ("--gifinterlace"):
                self.GIF.set_interlace()
            elif opt in ("-h", "--help"):      
                command_line_help()                      
                sys.exit()     
            elif opt in ("-m", "--mode"): 
                self.Tester.set_mode(arg)
            elif opt in ("--minquality"): 
                self.Tester.set_min_quality(arg)
            elif opt in ("--maxsize"): 
                self.Tester.set_max_size(arg)
            elif opt in ("-o", "--outdir"):
                if arg[-1] != "/":
                    arg = arg + "/"
                self.SaveDir = arg
            elif opt in ("-t", "--tempdir"):
              if(arg[-1]!="/"):
                arg=arg+"/"
              self.TempDir=arg
        
        for arg in args:
            if run_bash("[ -d " + arg + " ] && echo \"True\"") == "True":
                self.add_dir(arg)
            elif run_bash("[ -f " + arg + " ] && echo \"True\"") == "True":
                self.addFile(arg)
