# Copyright (C) 2009 Ferdinand Majerech
# This file is part of IMGCrush
# For conditions of distribution and use, see copyright notice in __init__.pyfrom util import *

from tempimage import *
from imagesettings import *
from referencefile import *

class JPGProcessor:
    """Handles JPG file generation and it's settings.
    """
    def __init__(self, tempdir):
        """Constructor.

        Data members:
        MinQuality : Minimal JPG quality to use. Can be int 0-100.
        Step       : Step to increase quality by when generating files.
        TempDir    : Directory to use to store generated temporary files.
        Interlace  : Generate interlaced JPG images ?
        Optimize   : Generate images with optimized JPG compression?
        SFactors   : Subsampling factors to use when generating images.
                     "standard" means common factors 1x1,2x1 and 2x2 are used.
                     "all" means that all available factors are used.
                     Otherwise this is a string of comma separated factors
                     in axb format.
                     Available factors are:
                     1x1 (no subsampling)
                     1x2
                     1x4
                     2x1
                     2x2
                     2x4
                     4x1
                     4x2
        """
        self.MinQuality = 50
        self.Step = 10
        self.TempDir = tempdir
        self.Interlace = False
        self.Optimize = False
        self.SFactors = "standard"
    
    def set_min_quality(self, q):
        """Sets minimum quality to use
        """
        self.MinQuality = valid_number(q, 50, "--jpgqual")
    
    def set_step(self, s):
        """Sets step to increase quality by.
        """
        self.Step = valid_number(s, 50, "--jpgqual")
    
    def set_interlace(self):
        """Turns interlacing on.
        """
        self.Interlace = True

    def set_optimize(self):
        """Turns optimized compression on.
        """
        self.Optimize = True

    def set_sfactors(self, f):
        """Sets subsampling factors to use.
        """
        if f in ("standard", "all"):
            self.SFactors = f
            return
        for fac in f.split(","):
            if fac not in ("1x1", "1x2", "1x4", "2x1", "2x2", "2x4", "4x1",\
                           "4x2"):
                print "WARNING: wrong value passed by --jpgsampfactors :\
                       using default"
                return
        self.SFactors = f

    def get_sfactors(self):
        """Parses stored subsampling factor data and returns a list of factors.
        """
        if(self.SFactors == "standard"):
            return ["1x1", "2x1", "2x2"]
        elif(self.SFactors == "all"):
            return ["1x1", "1x2", "1x4", "2x1", "2x2", "2x4", "4x1", "4x2"]
        else:
            return self.SFactors.split(",")
    
    def process_jpg(self, reffile, filenoext):
        """Generates temporary jpg files and returns a list of them.
        """
        print "\nprocessing jpg"
        sfactorlist = self.get_sfactors()
        outfilelist = []
        quality = self.MinQuality
        optimvals = [False]
        interlvals = [False]
        if self.Optimize:
            optimvals.append(True)
        if self.Interlace:
            interlvals.append(False)
        while True:
            for opt in optimvals:
                for inter in interlvals:
                    for factor in sfactorlist:
                        outfilename = self.TempDir + filenoext + "_" + factor\
                                      + str(inter) + str(opt) + str(quality)\
                                      + ".jpg";
                        outfilelist.append(TempImage(outfilename, "jpg", 
                                           JPGSettings(quality, inter,
                                           opt, factor), reffile))
            if quality == 100:
                break
            quality = quality + self.Step
            if quality > 100:
                quality = 100  
        return outfilelist
