# Copyright (C) 2009 Ferdinand Majerech
# This file is part of IMGCrush
# For conditions of distribution and use, see copyright notice in __init__.pyfrom util import *

from tempimage import *
from imagesettings import *
from referencefile import *
     
class PNGProcessor:
    """Handles PNG file generation and it's settings.
    """

    def __init__(self, tempdir):
        """Constructor.

        Data members:
        MinLevel     : Minimum compression level to use. Can be int 0-9.
        Filters      : PNG filters to use. String of numbers representing 
                       filters, for instance "125" for filters 1, 2 and 5.
                       Available filters:
                       0 : none
                       1 : sub
                       2 : up
                       3 : average
                       4 : Paeth
                       5 : Adaptive when not palleted and level more than 5
                       6 : Adaptive
        MinPalDepth  : Minimum bit depth to use for paletted images. 
                       e.g. bit depth 4 means there is maximum of 16 colors
                       in the palette. Can be 0, 1, 2, 4 or 8. 0 means paletted
                       images are not generated.
        MinGrayDepth : Minimum bit depth to use for grayscale images. 
                       e.g. bit depth 4 means there is maximum of 16 levels of
                       gray. Can be 0, 2, 3, 4 or 8. 0 means grayscale images
                       are not generated.
        BitMap       : Generate bitmaps ?
        Interlace    : Generate interlaced images?
        TempDir      : Directory to save generated temporary files to.
        """
        self.MinLevel = 7
        self.Filters = "06"
        self.MinPalDepth = 0
        self.MinGrayDepth = 0
        self.BitMap = False
        self.Interlace = False
        self.TempDir = tempdir
   
    def set_min_level(self, l):
        """Sets minimum compression level to use.
        """
        self.MinLevel = valid_number(q, 7, "--pnglev",0,9)
  
    def set_filters(self, filters):
        """Sets PNG filters to use.
        """
        self.Filters = ""
        for f in filters:
            if f in ("0", "1", "2", "3", "4", "5", "6")\
                 and (self.Filters.find(f) == -1):
                self.Filters = self.Filters + f
        if(self.Filters == ""):
            print "WARNING: wrong value passed by --pngfilters : using default"
            self.Filters = "15"

    def set_min_pal_depth(self, d):
        """Sets minimum bit depth to use with paletted images.
        """
        try:
            if(int(d) in (1, 2, 4, 8)):
                self.MinPalDepth = int(d)
            elif(int(d)==0):
                pass
            else:
                print "WARNING: wrong value passed by --pngpal : using default"
        except ValueException:
            print "WARNING: wrong value passed by --pngpal : using default"
    
    def set_min_gray_depth(self, d):
        """Sets the minimum bit depth to use with grayscale images.
        """
        try:
            if(int(d) in (2, 3, 4, 8)):
                self.MinGrayDepth = int(d)
            elif(int(d)==0):
                pass
            else:
                print "WARNING: wrong value passed by --pnggray : using default"
        except ValueException:
            print "WARNING: wrong value passed by --pnggray : using default"
    
    def set_bit_map(self):
        """Turns generation of bitmaps on.
        """
        self.BitMap = True
    
    def set_interlace(self):
        """Turns generation of interlaced images on.
        """
        self.Interlace = True
    
    def png_name(self, filenoext, level, filter, mode, depth, interlace):
        """Generates a filename for generated png based on it's settings.
        """
        addstr = ""
        if mode == "truecolor":
            addstr = addstr + "__lossless__"
        return self.TempDir + filenoext + addstr + str(interlace) + mode +\
               str(level) + filter + str(depth) + ".png"
     
    def png_helper(self, level, filter, interlace, reffile, filenoext):
        """Inner part of PNG generation.

        Taken out of inner loop of process_png.
        This generates PNG files using specified settings in all allowed
        modes, e.g. truecolor, paletted, grayscale, bitmap.
        """
        #truecolor image
        outfilename = self.png_name(filenoext, level, filter, "truecolor", 8,
                                    interlace)
        outfilelist = [TempImage(outfilename, "png", PNGSettings(level, filter\
                       , "truecolor", 8, interlace), reffile)]
        #paletted image/s
        if self.MinPalDepth:
            depth = self.MinPalDepth
            while depth <= 8:
                outfilename = self.png_name(filenoext, level, filter, 
                                            "palette", depth, interlace)
                outfilelist.append(TempImage(outfilename, "png", 
                                   PNGSettings(level, filter, "palette",
                                   depth, interlace), reffile))
                depth=depth*2
        #grayscale image/s
        if self.MinGrayDepth:
            depthlist=[2, 3, 4, 8]
            #find the index of given mindepth in list and start from there
            for d in depthlist[depthlist.index(self.MinGrayDepth):]:
                outfilename = self.png_name(filenoext, level, filter, "gray",
                                            d, interlace)
                outfilelist.append(TempImage(outfilename, "png",
                                   PNGSettings(level, filter, "gray", d,
                                   interlace), reffile))
        #bitmap image
        if self.BitMap:
            outfilename = self.png_name(filenoext, level, filter, "bitmap",
                                        depth, interlace)
            outfilelist.append(TempImage(outfilename, "png", 
                               PNGSettings(level, filter, "bitmap", 1, 
                               interlace), reffile))
        return outfilelist                   
      
    def process_png(self, reffile, filenoext):
        """Generates temporary png files and returns a list of them.
        """
        outfilelist = []
        print "\nprocessing png"
        for level in range(self.MinLevel, 10):
            for f in self.Filters:
                outfilelist.extend(self.png_helper(level, f, False, reffile,
                                   filenoext))
                if(self.Interlace):
                    outfilelist.extend(self.png_helper(level, f, True,
                                       reffile,filenoext))
        return outfilelist 
