# Copyright (C) 2009 Ferdinand Majerech
# This file is part of IMGCrush
# For conditions of distribution and use, see copyright notice in __init__.pyimport sys

import getopt
import subprocess
from math import *

def clamp(num, minval, maxval):
    """Clamps the given number by given range.

    If the number is smaller than minimum, the minimum is returned,
    it it's larger than maximum, the maximum is returned,
    otherwise, the unchanged number is returned.
    """

    if(num < minval):
        return minval
    elif(num > maxval):
        return maxval
    else:                                                         
        return num

def run_bash(cmd, err=False):
    """Executes Bash commands and returns their output.

    If err is true, output is returned from stderr instead of stdout.
    """
    p = subprocess.Popen(cmd, shell = True, stdout = subprocess.PIPE, 
                         stderr = subprocess.PIPE)
    if err:
        out = p.stderr.read().strip()
    else:
        out = p.stdout.read().strip()
    return out

def imagick_cmd(format, settings, filename, outfilename,
                sizeargs=[0, 0, False, False, "none"]):
    """Creates command line command for imagemagick of given data, e.g. format.
    
    Executes the command when it's complete.
    """
    cmd = "convert "
    width = sizeargs[0]
    height = sizeargs[1]
    fill = sizeargs[2]
    stretch = sizeargs[3]
    if width or height:
        cmd = cmd + "-resize "
        if width:
            cmd = cmd + str(width)
        if height:
            cmd = cmd + "x"+str(height)
        if width and height and stretch:
            cmd = cmd + "!"
        if width and height and fill:
            cmd = cmd + " -gravity Center -extent " + str(width) + "x"\
                      + str(height)
        cmd = cmd + " -strip "
    if format == "jpg":
        cmd = cmd + "-sampling-factor " + str(settings.SamplingFactor) + " "
        cmd = cmd + "-quality " + str(settings.Quality) + " "
        if settings.Interlace:
            cmd = cmd + "-interlace JPEG "
        if settings.Optimize:
            cmd = cmd + "-define jpeg:optimize-coding=true "
        else:
            cmd = cmd + "-define jpeg:optimize-coding=false "
    if format == "png":
        if settings.Interlace:
            cmd = cmd + "-interlace PNG "
        if settings.Mode == "palette":
            cmd = cmd + "-colors " + str(2 ** settings.BitDepth)\
                      + " -define png:color-type=Palette "
        if settings.Mode == "gray":
            cmd = cmd + "-colorspace Gray -colors "\
                      + str(2 ** settings.BitDepth)\
                      + " -define png:color-type=GrayScale "
        if settings.Mode == "bitmap":
            cmd = cmd + "-colors " + str(2) + " -define png:color-type=BiLevel "   
        cmd = cmd + "-quality " + str(settings.Level) + str(settings.Filter)\
                  + " "
    if format == "gif":
        if settings.Interlace:
            cmd = cmd + "-interlace GIF "
    cmd = cmd + filename + " " + outfilename
    subprocess.call(cmd, shell = True)

def valid_number(input, default, command="UNKNOWN", min=0, max=100, 
                 isfloat=False):
    """Validates input as int and returns int.

    If input is invalid, prints warning about given command passing
    wrong value and returns given default value.
    """
    try:
        if isfloat:
            return clamp(float(input), min, max)
        else:
            return clamp(int(input), min, max)
    except ValueError:
        print "WARNING: wrong value passed by ", command, " : using default."
        print "Min value: ", min, "Max value: ", max, "Type of value: ",\
              "float" if isfloat else "int"
        return default

def valid_number_str(input, default, command="UNKNOWN", min=0, max=100,
                     isfloat=False):
    """Validates input as int and returns string.

    If input is invalid, prints warning about given command passing
    wrong value and returns given default value.
    """
    return str(valid_number(input, default, command, min, max, isfloat))

def valid_number_str_suffix(input, default, sufx, command,  min, max, 
                            isfloat=False):
    """Validates input as int with suffix and returns string.

    If input is invalid, prints warning about given command passing
    wrong value and returns given default value.
    """
    return valid_number_str(input[ :-len(sufx)], default, command +
                            " with suffix " + sufx, min, max, isfloat) + sufx

def escape_chars(input, chars):
    """Puts escape characters before spaces in input.

    """
    linput = list(input)
    for c in chars:
        try:
            charpos = linput.index(c)
            inputpos = 0
            while True:
                linput.insert(charpos,"\\")
                inputpos = charpos+2
                charpos = linput[inputpos:].index(c)
        except ValueError:
            pass
    return "".join(linput)

def escape_chars_bash(input):
    """Escapes special bash characters.

    Used for command line arguments containing these characters.
    """
    return escape_chars(input, ["\\", " ", "(", ")", ";", "\"", "\'", "$", "!",\
                                "&", "<", ">", "|"] )
